"""
Interfaces
==========

An interface is the functional implementation for command line but usable from Python
code without tricks involved by command lines requirements like importing settings or
page modules.

"""
